
var events = require("../../lib/events")
, url = require("url")
, dgram = require("dgram")
, http = require("http")

module.exports = SSDP

function SSDP(name) {
	var ssdp = this
	, parsed = url.parse(name, true)

	ssdp.roots = {}
	ssdp.handleMessage = ssdp.handleMessage.bind(ssdp)
	ssdp.opts = Object.assign({
		hostname: "239.255.255.250",
		port: 1900,
		service: "ssdp:all",
		mx: 3,
		ttl: 4
	}, parsed.query, parsed)

	ssdp.openSocket()
}

SSDP.prototype = {
	openSocket: function() {
		if (this.socket) {
			console.log("Close existing socket")
			this.socket.close()
		}
		const opts = this.opts
		const socket = this.socket = dgram.createSocket({type: "udp4", reuseAddr: true})

		socket.on("error", function(err) {
			console.error("ssdp error: " + err)
		})
		socket.on("message", this.handleMessage)

		socket.bind(+opts.port, () => {
			const a = socket.address()
			console.log("SSDP listening on " + a.address + ":" + a.port)
			socket.addMembership(opts.hostname)
			socket.setMulticastTTL(+opts.ttl || 2)
			this.emit("listening")
		})
	}

	handleMessage(buf, rinfo) {
		const httpRe = /^(?:(\S+)\s+(.+)\s+HTTP\/\d\.\d|HTTP\/\d\.\d (\d+) (.*))$/
		const headerRe = /^([^:]+):\s*(.*)$/

		let lines = buf.toString().split("\r\n")
		let match = httpRe.exec(lines.shift())

		if (!match) {
			throw Error("Invalid response:\n" + buf.toString())
		}

		let headers = {}
		for (let line, i = 0; line = headerRe.exec(lines[i++]); ) {
			headers[line[1].toLowerCase()] = line[2]
		}

		if (match[1] == "NOTIFY") {
			let junks = (headers.usn || headers.nt).split(":")
			let uuid = junks[1]

			if (!headers.nts) {
				throw "SSDP NOTIFY without NTS header"
			}
			let root = this.roots[uuid] || (this.roots[uuid] = {
				uuid: uuid, locations: {}, services: {},
				devices: {}, headers: {}
			})

			Object.assign(root.headers, headers)

			root.alive = headers.nts == "ssdp:alive"
			this.emit("notify", headers, root)
		} else if (match[1] == "M-SEARCH") {
			if (!headers.man || !headers.mx || !headers.st) return
			this.emit("m-search", headers)

		} else {
			this.emit("response", headers, rinfo)
		}


	}

	search(service) {
		const opts = this.opts
		const buf = Buffer.from([
			"M-SEARCH * HTTP/1.1",
			"HOST: " + opts.host,
			'MAN: "ssdp:discover"',
			"MX: " + opts.mx,                     // Maximum wait time in seconds. Should be between 1 and 120 inclusive.
			"ST: " + (service || opts.service),
			""
		].join("\r\n"))

		console.log("search " + buf.toString())
		this.socket.send(buf, 0, buf.length, opts.port, opts.hostname)
	}
}

class SSDP extends EventEmitter {
	constructor(name) {
		super()
	}

	openSocket() {
		if (this.socket) {
			console.log("Close existing socket")
			this.socket.close()
		}
		const opts = this.opts
		const socket = this.socket = dgram.createSocket({type: "udp4", reuseAddr: true})

		socket.on("error", function(err) {
			console.error("ssdp error: " + err)
		})
		socket.on("message", this.handleMessage)

		socket.bind(+opts.port, () => {
			const a = socket.address()
			console.log("SSDP listening on " + a.address + ":" + a.port)
			socket.addMembership(opts.hostname)
			socket.setMulticastTTL(+opts.ttl || 2)
			this.emit("listening")
		})
	}

	handleMessage(buf, rinfo) {
		const httpRe = /^(?:(\S+)\s+(.+)\s+HTTP\/\d\.\d|HTTP\/\d\.\d (\d+) (.*))$/
		const headerRe = /^([^:]+):\s*(.*)$/

		let lines = buf.toString().split("\r\n")
		let match = httpRe.exec(lines.shift())

		if (!match) {
			throw Error("Invalid response:\n" + buf.toString())
		}

		let headers = {}
		for (let line, i = 0; line = headerRe.exec(lines[i++]); ) {
			headers[line[1].toLowerCase()] = line[2]
		}

		if (match[1] == "NOTIFY") {
			let junks = (headers.usn || headers.nt).split(":")
			let uuid = junks[1]

			if (!headers.nts) {
				throw "SSDP NOTIFY without NTS header"
			}
			let root = this.roots[uuid] || (this.roots[uuid] = {
				uuid: uuid, locations: {}, services: {},
				devices: {}, headers: {}
			})

			Object.assign(root.headers, headers)

			root.alive = headers.nts == "ssdp:alive"
			this.emit("notify", headers, root)
		} else if (match[1] == "M-SEARCH") {
			if (!headers.man || !headers.mx || !headers.st) return
			this.emit("m-search", headers)

		} else {
			this.emit("response", headers, rinfo)
		}


	}

	search(service) {
		const opts = this.opts
		const buf = Buffer.from([
			"M-SEARCH * HTTP/1.1",
			"HOST: " + opts.host,
			'MAN: "ssdp:discover"',
			"MX: " + opts.mx,                     // Maximum wait time in seconds. Should be between 1 and 120 inclusive.
			"ST: " + (service || opts.service),
			""
		].join("\r\n"))

		console.log("search " + buf.toString())
		this.socket.send(buf, 0, buf.length, opts.port, opts.hostname)
	}
}

var ssdp = new SSDP("ssdp://239.255.255.250:1900/?ttl=1")
.on("listening", function() {
	//this.socket.unref()
	this.search()
})
.on("notify", function(headers) {
	console.log("notify", headers)

})
.on("notify", function(headers) {
	console.log("notify", headers)

})

/*
//

USN - Unique Service Name
	uuid:device-UUID
		Sent once for every device, root or embedded. Device UUID specified by UPnP vendor.
	uuid:device-UUID::upnp:rootdevice
		Sent once for root device. Device UUID specified by UPnP vendor.
	uuid:device-UUID::urn:schemas-upnp-org:device:deviceType:v
		Sent once for every device, root or embedded. Device UUID specified by UPnP vendor. Device type and
		version defined by UPnP Forum working committee. Specifies the highest supported version of the device
		type.
	uuid:device-UUID::urn:schemas-upnp-org:service:serviceType:v
		Sent once for every service. Device UUID specified by UPnP vendor. Service type and version defined by UPnP
		Forum working committee. Specifies the highest support version of the service type.
	uuid:device-UUID::urn:domain-name:device:deviceType:v
		Sent once for every device, root or embedded. Device UUID, domain name, device type and version defined
		by UPnP vendor. Specifies the highest supported version of the device type. Period characters in the domain
		name must be replaced by hyphens in accordance with RFC 2141.
	uuid:device-UUID::urn:domain-name:service:serviceType:v
		Sent once for service. Device UUID, domain name, service type and version defined by UPnP vendor. Specifies
		the highest supported version of the device type. Period characters in the domain name must be replaced by
		hyphens in accordance with RFC 2141.

ST - Search Target. Must be one of the following. (cf. NT header in NOTIFY with ssdp:alive above.) Single URI.
	ssdp:all
		Search for all devices and services.
	upnp:rootdevice
		Search for root devices only.
	uuid:device-UUID
		Search for a particular device. Device UUID specified by UPnP vendor.
	urn:schemas-upnp-org:device:deviceType:v
		Search for any device of this type. Device type and version defined by UPnP Forum working committee.
	urn:schemas-upnp-org:service:serviceType:v
		Search for any service of this type. Service type and version defined by UPnP Forum working committee.
	urn:domain-name:device:deviceType:v
		Search for any device of this type. Domain name, device type and version defined by UPnP vendor. Period
		characters in the domain name must be replaced with hyphens in accordance with RFC 2141.
	urn:domain-name:service:serviceType:v
		Search for any service of this type. Domain name, service type and version defined by UPnP vendor. Period
		characters in the domain name must be replaced with hyphens in accordance with RFC 2141.




DLNA_RENDERER = 'urn:schemas-upnp-org:device:MediaRenderer:1'
DLNA_SERVER = 'urn:schemas-upnp-org:device:MediaServer:1'
GATEWAY = 'urn:schemas-upnp-org:device:InternetGatewayDevice:1'
PANASONIC_VIERA = 'urn:panasonic-com:device:p00RemoteController:1'
ROKU = 'roku:ecp'
SONOS = 'urn:schemas-upnp-org:device:ZonePlayer:1'
SONY_NEX = 'urn:schemas-sony-com:service:ScalarWebAPI:1'
WAN_IP = 'urn:schemas-upnp-org:service:WANIPConnection:1'
WAN_PPP = 'urn:schemas-upnp-org:service:WANPPPConnection:1'

*/

