
/* litejs.com/MIT-LICENSE.txt */


El.drag = function(handler, node, next, params) {
	var mouseStart, dragStart, topPos, leftPos

	if (typeof handler == "string") {
		handler = El.find(handler)
	}
	if (typeof node == "string") {
		node = El.find(node)
	}

	if (!node) node = handler

	var parent = node.parentNode

	Event.touchAsMouse(handler)
	El.on(handler, "mousedown", drag)

	function drag(e) {
		if (e.button) return

		var target = e.target || e.srcElement
		, tagName = target && target.tagName
		, top = node.style.top

		if (
			!tagName ||
			tagName === "INPUT" ||
			tagName === "TEXTAREA" ||
			tagName === "SELECT" ||
			El.hasClass(target, "no-drag")
			){
			return
		}

		mouseStart = El.mouse(e)

		dragStart = {
			top: parseInt(top, 10) || 0,
			left: parseInt(node.style.left, 10) || 0
		}
		if (top && top.indexOf("%") > -1) {
			dragStart.top  *= parent.offsetHeight / 100
			dragStart.left *= parent.offsetWidth / 100
		}

		El.on(document, "mouseup", dragEnd)
		El.on(document, "mousemove", dragMove)
		if (e.preventDefault) e.preventDefault()
		return false
	}

	function dragMove(e) {
		var mouse = El.mouse(e)
		, leftMax = parent.clientWidth >> 2
		, leftMin = leftMax - node.offsetWidth
		, topMax = parent.clientHeight >> 2
		, topMin = topMax - node.offsetHeight
		leftMax *= 3
		topMax *= 3

		leftPos = dragStart.left - mouseStart.left + mouse.left
		topPos  = dragStart.top  - mouseStart.top  + mouse.top

		if (leftPos > leftMax) leftPos = leftMax
		else if (leftPos < leftMin) leftPos = leftMin
		if (topPos > topMax) topPos = topMax
		else if (topPos < topMin) topPos = topMin

		node.style.left = leftPos + "px"
		node.style.top = topPos + "px"
		return false
	}

	function dragEnd() {
		if (next && Math.abs(topPos - dragStart.top) + Math.abs(leftPos - dragStart.left) > 5) {
			next()
		}
		El.off(document, "mousemove", dragMove)
		El.off(document, "mouseup", dragEnd)
		return false
	}
}


